@@ -12,10 +12,39 @@
 # nolint end
 #' @export
 #' @concept dataset
+#' @family subnational
 #' @examples
 #' \dontrun{
-#' region <- UK$new(level = "1", verbose = TRUE, steps = TRUE, get = TRUE)
+#' # setup a data cache
+#' start_using_memoise()
+#'
+#' # download, clean and process level 1 UK data with hospital admissions
+#' region <- UK$new(level = "1", nhsregions = TRUE)
 #' region$return()
+#'
+#' # initialise level 2 data
+#' utla <- UK$new(level = "2")
+#'
+#' # download UTLA data
+#' utla$download()
+#'
+#' # clean UTLA data
+#' utla$clean()
+#'
+#' # inspect available level 1 regions
+#' utla$available_regions(level = "1")
+#'
+#' # filter data to the East of England
+#' utla$filter("East of England")
+#'
+#' # process UTLA data
+#' utla$process()
+#'
+#' # return processed and filtered data
+#' utla$return()
+#'
+#' # inspect all data steps
+#' utla$data
 #' }
 UK <- R6::R6Class("UK",
   inherit = DataClass,
@@ -40,7 +69,14 @@ UK <- R6::R6Class("UK",
     #' @field level_data_urls List of named links to raw data that are level
     #' specific.
     level_data_urls = list(
-      "1" = list("nhs_base_url" = "https://www.england.nhs.uk/statistics")
+      "1" = list(
+        # A dynamic URL for NHS data from 7 April 2021 to now
+        "nhs_recent_url" = "https://www.england.nhs.uk/statistics",
+        # A stable URL for data from August 2020 - April 06 2021
+        # nolint start
+        "nhs_archive_url" = "https://www.england.nhs.uk/statistics/wp-content/uploads/sites/2/2021/04/COVID-19-daily-admissions-and-beds-20210406-1.xlsx"
+      )
+      # nolint end
     ),
     #' @field source_data_cols existing columns within the raw data
     source_data_cols = list(
@@ -121,7 +157,7 @@ UK <- R6::R6Class("UK",
       if (!is.null(self$release_date)) {
         self$data$clean <- mutate(
           self$data$clean,
-          release_date <- self$release_date
+          release_date = self$release_date
         )
       }
       # get NHS data if requested
@@ -308,6 +344,8 @@ UK <- R6::R6Class("UK",
     #' @description Download NHS data for level 1 regions
     #' Separate NHS data is available for "first" admissions, excluding
     #' readmissions. This is available for England + English regions only.
+    #' Data are available separately for the periods 2020-08-01 to 2021-04-06,
+    #' and 2021-04-07 - present.
     # nolint start
     #'   See: \url{https://www.england.nhs.uk/statistics/statistical-work-areas/covid-19-hospital-activity/}
     # nolint end
@@ -317,9 +355,11 @@ UK <- R6::R6Class("UK",
     #' @source \url{https://coronavirus.data.gov.uk/details/download}
     # nolint end
     #' @importFrom lubridate year month
-    #' @importFrom readxl read_excel cell_limits
+    #' @importFrom readxl cell_limits
     #' @importFrom dplyr %>%
     download_nhs_regions = function() {
+
+      # Some general set up
       if (is.null(self$release_date)) {
         self$release_date <- Sys.Date() - 1
       }
@@ -334,8 +374,10 @@ UK <- R6::R6Class("UK",
         admissions, which excludes readmissions. This is available for
         England and English regions only."
       )
-      nhs_url <- paste0(
-        self$data_urls[["nhs_base_url"]],
+
+      # 1. Data from 7 April 2021 to now:
+      nhs_recent_url <- paste0(
+        self$data_urls[["nhs_recent_url"]],
         "/wp-content/uploads/sites/2/",
         year(self$release_date), "/",
         ifelse(month(self$release_date) < 10,
@@ -346,19 +388,30 @@ UK <- R6::R6Class("UK",
         gsub("-", "", as.character(self$release_date)),
         ".xlsx"
       )
-      tmp <- file.path(tempdir(), "nhs.xlsx")
-      download.file(nhs_url,
-        destfile = tmp,
-        mode = "wb", quiet = !(self$verbose)
+
+      recent <- download_excel(nhs_recent_url,
+        "nhs_recent.xlsx",
+        verbose = self$verbose,
+        transpose = TRUE,
+        sheet = 1,
+        range = cell_limits(c(28, 2), c(36, NA))
       )
-      nhs <- suppressMessages(
-        read_excel(tmp,
-          sheet = 1,
-          range = cell_limits(c(28, 2), c(36, NA))
-        ) %>%
-          t()
+
+      # 2. Data for August 2020 to 7 April 2021
+      archive <- download_excel(self$data_urls[["nhs_archive_url"]],
+        "nhs_archive.xlsx",
+        verbose = self$verbose,
+        transpose = TRUE,
+        sheet = 1,
+        range = cell_limits(c(28, 2), c(36, NA))
       )
-      return(as.data.frame(nhs))
+
+
+      # 3. Join archive and recent data, still in raw format
+      dt <- rbind(archive, recent)
+      dt <- dt[which(!rownames(dt) == "Name1"), ]
+
+      return(dt)
     },
 
     #' @description Add NHS data for level 1 regions
